import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.*;

public class Palette extends JPanel {
   private String titre;
   protected ZoneImage zone;
   
   public Palette(String titre, int hauteur, ZoneImage zone) {
      this.titre = titre;
      this.zone = zone;
      this.setPreferredSize(new Dimension(300, hauteur));
      this.setBorder(BorderFactory.createRaisedBevelBorder());      
      this.add(new JLabel("                 "));
   }
   protected void paintComponent(Graphics g) {
      Graphics2D zone = (Graphics2D) g;
      zone.setPaint(Color.orange);
      zone.fill(new Rectangle2D.Double(0, 0, 300, 25));
      zone.setPaint(Color.yellow);
      zone.setFont(new Font("SansSerif", Font.BOLD+Font.ITALIC, 16));
      zone.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      zone.drawString(titre, 10, 20);
   }
}

class PaletteNiveaux extends Palette {
   public PaletteNiveaux(ZoneImage image) {
      super("Rglage des niveaux", 220, image);
      try {
         this.add(new Rglage("Intensit : Assombrir..............................Eclaircir", image, "intensitLumineuse", -70, 70));
         this.add(new Rglage("Noir profond (pour augmenter le contraste)", image, "rglerDcalageNoir", 0, 70));
         this.add(new Rglage("Blanc le plus intense (pour le contraste)", image, "rglerDcalageBlanc", 0, 70));
      }
      catch (Exception e) {
         e.printStackTrace();
      }
   }
}

class PaletteAttnuation extends Palette {
   public PaletteAttnuation(ZoneImage image) {
      super("Attnuation des extrmes", 160, image);
      try {
         this.add(new Rglage("Eclaircir les parties les plus sombres", image, "attnuationFortesDensits", 0, 70));
         this.add(new Rglage("Diminuer les hautes lumires (parties claires)", image, "diminutionHautesLumires", 0, 70));
      }
      catch (Exception e) {
         e.printStackTrace();
      }
   }
}

class PaletteAccentuation extends Palette  {
   private JPanel saturation = new JPanel();
   private BoutonSaturation plus;
   private BoutonSaturation moins; 

   public PaletteAccentuation(ZoneImage image) {
      super("Accentuations", 160, image);
      try {
         this.add(new Rglage("Contraste", image, "rglerContraste", -20, 40));
         saturation.setBorder(BorderFactory.createTitledBorder("Saturation (Augmenter les couleurs)"));
         moins = new BoutonSaturation("-", image, -10);
         plus = new BoutonSaturation("+", image, 10);
         saturation.add(moins);
         saturation.add(plus);
         this.add(saturation);
      }
      catch (Exception e) {
         e.printStackTrace();
      }
   }
}

class BoutonSaturation extends JButton implements ActionListener {
   private ZoneImage zone;
   private int valeur;
   
   public BoutonSaturation(String libell, ZoneImage zone, int valeur) {
      super(libell);
      this.zone = zone;
      this.valeur = valeur;
      setPreferredSize(new Dimension(128, 25));
      addActionListener(this);
   }
   public void actionPerformed(ActionEvent e) {
      zone.saturation(valeur);
   }
}

class PaletteHistogramme extends Palette {
   private Histogramme histogramme;
   
   public PaletteHistogramme(ZoneImage zone) {
      super("Histogramme", 220, zone);
      histogramme = new Histogramme(zone);
      zone.setHistogramme(histogramme);
      this.add(histogramme);
   }
}